
/*============================================================================
 Name        : libuserCmd.c
 Author      : atao
 Date  	     : 2015.9
 Copyright   : ZKTeco.
 Description : Linux C
 ============================================================================*/
#include "serial.h"
#include "libtype.h"
#include "command_fun.h"
#include "libuserCmd.h"
#include "zkfpi.h"
#include <pthread.h>

#define		SDKVERSION			"2.0.3"
#define 	USBPID				0x0121
//#define 	USBPID				0x0824
#define 	USBVID				0x1B55
#define 	TIMEOUT				9000

TExtDataBuf ExtDataBuffer;
U8 *DATABUFF = NULL;
U8 *DATA_pro = NULL;
U8 *DATA_end = NULL;
INT SetTime_loop = -1;
INT Err_flag = 0;
static INT Dev_num = -1;

static VOID SetExtDataBuffer(INT size, U8 *buf)
{
	if(DEBUG)
	{
		fprintf(sdklog, "get image size %d bytes\n", size);
	}
	ExtDataBuffer.buffer=buf;
	ExtDataBuffer.bufferSize=size;
	ExtDataBuffer.bufEnd=buf +size;
	ExtDataBuffer.bufPtr=buf;
	ExtDataBuffer.Index=0;
	ExtDataBuffer.PacketNum=0;
	ExtDataBuffer.PacketLen=0;
}

INT Connect(CHAR *dev, INT type)
{
	INT ret = 0;

	OpenDebugFile();

	switch(type)
	{
		case DEV_USB:			//usb init
			Dev_num = DEV_USB;
			INIT("USB", DEV_USB);
			ret = usb.init(USBVID, USBPID);
			if(ret)
			{
				if((ret = ZKFPI_Open(0)))
				{
					//clear the USB buffer data
					Clear();
					ret = GetStatus();
				}
			}
			break;
		case DEV_232:
			Dev_num = DEV_232;
			ret = INIT(dev, DEV_232);//ff232 init
			if(ret)
			{
				ret = GetStatus();
			}
			break;
		default :
			break;
	}
	return ret;
}

INT Final(VOID)
{
	if(DATABUFF != NULL)
	{
		free(DATABUFF);
		DATABUFF = NULL;
	}
	if(DataPackage != NULL)
	{
		free(DataPackage);
		DataPackage = NULL;
	}
	if(Dev_num == DEV_232)
	{
		ff232.free();		//close 232
	}
	else if(Dev_num == DEV_USB)
	{
		ZKFPI_Close();		//close usb
	}

	Dev_num = -1;

	CloseDebugFile();

	return SUCCESS;
}

/********************* Parameter select *********************/
static INT ParameterSelect(INT flag)
{
	INT i = 0;
	INT parameter_value[] =
	{
			SID_SAVE_LOG,
			SID_AUTO_ACK,
			SID_TIMEOUT,
			SID_FW_VER,
			SID_BAUDRATE,
			SID_ENROLL_FP,
			SID_FP_COUNT,
			SID_USER_COUNT,
			SID_LOG_NUM,
			SID_LOG_COUNT,
			SID_BUILD_NUM,
			SID_MODULE_ID,
			SID_FP_FORMAT,
			SID_MODULE_IDENTIFY,
			SID_GPIO_LEVEL
	};
	for(i = 0; i < sizeof(parameter_value); i++)
	{
		if(flag == parameter_value[i])
		{
			break;
		}
	}
	if(i >= sizeof(parameter_value))
	{
		//Parameter err...
		return FAILURE;
	}

	return SUCCESS;
}

/****************** User command *****************/

INT EnrollUserByScan(INT userID)
{
	INT count = 0;
	INT ret = -1;
	INT Param = userID;
	INT Flag = 0;

	INT len = 0;

	
	//send data
	ret = Cmd.SendCmd(MD_ENROLL_SCAN, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	do
	{
		//recv data
		ret = Cmd.RecvCmd(MD_ENROLL_SCAN, &Flag, &Param, NULL, &len, TIMEOUT);
		if(ret < 0)
		{
			return FAILURE;
		}
		count++;
		if(DEBUG)
		{
			fprintf(sdklog, "count = %d\r\n", count);
		}
	}while((0x62 == Flag) && (count < 3));

	//Operation OK?
	if((Flag != FLAG_OK))
	{
		Err_flag = Flag;
		return FAILURE;
	}
	printDebug(1, "[ ok ]: [01]EnrollUserByScan.....OK!");
	return SUCCESS;
}

INT GetStatus(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;

	//send data
	ret = Cmd.SendCmd(MD_SYS_STATUS, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_SYS_STATUS, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	// GetStatus.....OK!
	return SUCCESS;
}

INT ReadAllLogs(INT *userID,  CHAR *event, CHAR *verified, U32 *date, U32 *time, CHAR *reserved)
{
	INT ret = 0;
	INT dataSize = 0;
	static INT loop = 0;
	tLog data;

	if(0 == loop)
	{
		if(!GetAllLogs(&dataSize))
		{
			if(DATABUFF != NULL)
			{
				free(DATABUFF);
				DATABUFF = NULL;
			}
			return FAILURE;
		}
	}

	if(DATA_pro < DATA_end)
	{
		memcpy(&data, (pLog)DATA_pro, sizeof(tLog));
		DATA_pro += sizeof(tLog);
		ret = 1;
		loop = 1;
	}
	else
	{
		//Recv Record Over!
		loop = 0;
		ret = 0;
	}

	//return user data
	memcpy(reserved, data.Reserved, sizeof(data.Reserved));
	*time = data.Time;
	*date = data.Date;
	*verified = data.verified;
	*event = data.Event;
	*userID = data.userID;

	if((0 == loop) && (DATABUFF != NULL))
	{
		free(DATABUFF);
		DATABUFF = NULL;
	}
	return ret;
}

INT GetAllLogs(INT *dataSize)
{
	INT ret = -1;
	INT Param = MAX_DATA;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;

	//1. send command
	ret = Cmd.SendCmd(MD_LOAD_LOG_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_LOAD_LOG_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	size = Param;

	if(Flag == FLAG_OK)
	{
		//2. recv extData
		DATABUFF = malloc(size);
		if(DATABUFF == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(DATABUFF, 0, size);
		SetExtDataBuffer(size, DATABUFF);
		RecvExtDataPro(MD_LOAD_LOG_X, TIMEOUT);		//RECV OK!
	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		//Operation failure...
		return FAILURE;
	}
	*dataSize = size;
	DATA_pro = DATABUFF;
	DATA_end = DATABUFF + size;

	return SUCCESS;
}

INT ReadAllUser(INT *userID,  CHAR *name,  CHAR *password, U16 *secLevel,
		U32 *PIN2, U8 *privilege, U8 *fingerprintNum, U8 *Card)
{
	INT ret = 0;
	INT dataSize = 0;
	static INT loop = 0;
	tUser data;

	if(0 == loop)
	{
		if(!GetAllUsers(&dataSize))
		{
			if(DATABUFF != NULL)
			{
				free(DATABUFF);
				DATABUFF = NULL;
			}
			return FAILURE;
		}
	}

	if(DATA_pro < DATA_end)
	{
		memcpy(&data, (pUser)DATA_pro, sizeof(tUser));
		DATA_pro += sizeof(tUser);
		ret = 1;
		loop = 1;
	}
	else
	{
		//Read User Over!
		printDebug(1, "[ ok ]: [17]ReadAllUser.....OK!");
		loop = 0;
		ret = 0;
	}
	//return user data
	memcpy(name, data.Name, sizeof(data.Name));
	memcpy(password, data.Password, sizeof(data.Password));
	memcpy(Card, data.Card, sizeof(data.Card));
	*secLevel = data.SecLevel;
	*PIN2 = data.PIN2;
	*privilege = data.Privilege;
	*fingerprintNum = data.Fpnum;
	*userID = data.userID;

	if(0 == loop)
	{
		if(DATABUFF != NULL)
		{
			free(DATABUFF);
			DATABUFF = NULL;
		}
	}
	return ret;
}

INT GetAllUsers(INT *dataSize)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;

	//1. send command
	ret = Cmd.SendCmd(MD_LOAD_USER_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_LOAD_USER_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	size = Param;

	if(Flag == FLAG_OK)
	{
		//2. recv extData
		DATABUFF = malloc(size);
		if(DATABUFF == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(DATABUFF, 0, size);
		SetExtDataBuffer(size, DATABUFF);
		RecvExtDataPro(MD_LOAD_USER_X, TIMEOUT);	//RECV OK!
	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	//3. data
	*dataSize = size;
	DATA_pro = DATABUFF;
	DATA_end = DATABUFF + size;
	return SUCCESS;
}

INT ReadTemplates(INT userID, INT index,  INT flag, U8 *data)
{
	INT ret = 0;
	INT dataSize = 0;
	static INT loop = 0;

	if(0 == loop)
	{
		if(!GetTemplates(userID, index, flag, &dataSize))
		{
			if(DATABUFF != NULL)
			{
				free(DATABUFF);
				DATABUFF = NULL;
			}
			printDebug(1, "[ err ]: GetTemplates err!");
			return FAILURE;
		}
	}

	if(DATA_pro < DATA_end)
	{
		memcpy(data, DATA_pro, TEMPLATES_SIZE);
		DATA_pro += TEMPLATES_SIZE;
		ret = 1;
		loop = 1;
	}
	else
	{
		//Recv Templates Over!
		loop = 0;
		ret = 0;
	}

	if((0 == loop) && (DATABUFF != NULL))
	{
		free(DATABUFF);
		DATABUFF = NULL;
	}
	return ret;
}

INT GetTemplates(INT userID, INT index,  INT flag, INT *dataSize)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;
	INT count = 0;

	if((flag == 0) || (flag == 1))
	{
		Flag = flag;
	}
	else
	{
		return FAILURE;
	}
	//index << 16 | user ID
	Param = ((index << 16) | (userID));

	//1. send command
	ret = Cmd.SendCmd(MD_READ_TMP_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_READ_TMP_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	size = Param & 0xff;
	count = (Param >> 8) & 0xff;
	if(count > 0)
	{
		size = TEMPLATES_SIZE * count;
	}

	if(Flag == FLAG_OK)
	{
		//2. recv extData
		DATABUFF = malloc(size);
		if(DATABUFF == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(DATABUFF, 0, size);
		SetExtDataBuffer(size, DATABUFF);
		RecvExtDataPro(MD_READ_TMP_X, TIMEOUT);//RECV OK!

	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	//3. data
	*dataSize = size;
	DATA_pro = DATABUFF;
	DATA_end = DATABUFF + size;

	return SUCCESS;
}

INT ReadAllTemplates(U8 *data)
{
	INT dataSize = 0;
	INT ret = 0;
	static INT loop = 0;

	if(0 == loop)
	{
		if(!GetAllTemplates(&dataSize))
		{
			if(DATABUFF != NULL)
			{
				free(DATABUFF);
				DATABUFF = NULL;
			}
			printDebug(1, "[ err ]: GetAllTemplates err!");
			return FAILURE;
		}
	}

	if(DATA_pro < DATA_end)
	{
		memcpy(data, DATA_pro, TEMPLATES_SIZE);
		DATA_pro += TEMPLATES_SIZE;
		loop = 1;
		ret = 1;
	}
	else if(DATA_pro == DATA_end)
	{
		//Recv Templates Over!
		//Operation is successful!
		loop = 0;
		ret = 1;
	}
	else
	{
		loop = -1;
		ret = 0;
	}

	if((0 == loop) || (-1 == loop) )
	{
		if(DATABUFF != NULL)
		{
			free(DATABUFF);		//memory free OK!
			DATABUFF = NULL;
		}
	}
	return ret;
}

INT GetAllTemplates(INT *dataSize)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;


	//1. send command
	ret = Cmd.SendCmd(MD_LOAD_TMP_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_LOAD_TMP_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	size = Param;
	if(Flag == FLAG_OK)
	{
		//2. recv extData
		DATABUFF = malloc(size);
		if(DATABUFF == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(DATABUFF, 0, size);
		SetExtDataBuffer(size, DATABUFF);
		RecvExtDataPro(MD_LOAD_TMP_X, TIMEOUT);	//RECV OK!
	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	//3. data
	*dataSize = size;
	DATA_pro = DATABUFF;
	DATA_end = DATABUFF + size;

	return SUCCESS;
}

INT ModifyUser(INT userID,  CHAR *name,  CHAR *password, U16 secLevel,
		U32 PIN2, U8 privilege,	U8 fingerprintNum, U8 *Card)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0x61;

	INT len = sizeof(tUser);
	tUser user;

	user.userID = userID;
	user.Fpnum = fingerprintNum;
	user.PIN2 = PIN2;
	user.Privilege = privilege;
	user.SecLevel = secLevel;
	memcpy(user.Name, name, sizeof(user.Name));
	memcpy(user.Password, password, sizeof(user.Password));
	memcpy(user.Card, Card, sizeof(user.Card));

	//send data
	ret = Cmd.SendCmd(MD_ADD_USER, Param, Flag, (U8 *)&user, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//recv data
	ret = Cmd.RecvCmd(MD_ADD_USER, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [19]ModifyUser.....OK!");
	return SUCCESS;
}

INT GetFingerImage(INT *width, INT *height, U8 * data)
{
	INT ret = -1;
	INT Param = 0;		//0 -> 4k
	INT Flag = 0;
	INT len = 0;


	SHORT Index = 0;
	SHORT PacketNum = 0;
	PImage image = NULL;
	U8 buff[1024];

	memset(buff, 0, sizeof(buff));
	image = (PImage)buff;

	//1. send command
	ret = Cmd.SendCmd(MD_SCAN_IMAGE_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_SCAN_IMAGE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. recv image data info
	if(Flag == FLAG_OK)
	{
		//recv data
		ret = Cmd.extRecvCmd(MD_SCAN_IMAGE_X, &Flag, &PacketNum, &Index, buff, &len, TIMEOUT);
		*width = image->width;
		*height = image->height;

		//fprintf(Fp, "width = %d    height = %d\r\n", *width, *height);
		if(DATABUFF != NULL)
		{
			free(DATABUFF);
			DATABUFF = NULL;
		}
		DATABUFF = (U8 *)malloc(image->img_len);
		if (NULL == DATABUFF)
		{
			//malloc Err
			Flag = ERR_DATA_ERROR;
			Cmd.extSendCmd(MD_SCAN_IMAGE_X, Flag, PacketNum, Index, NULL, 0);
			return FAILURE;
		}
		else
		{
			Flag = DATA_OK;
			ret = Cmd.extSendCmd(MD_SCAN_IMAGE_X, Flag, PacketNum, Index, NULL, 0);
		}

		//3. recv extData
		memset(DATABUFF, 0, image->img_len);
		SetExtDataBuffer(image->img_len, DATABUFF);
		if (RecvExtDataPro(MD_SCAN_IMAGE_X, TIMEOUT) > 0)	//image OK!
		{
			memcpy(data, DATABUFF, (image->img_len));
			Flag = FLAG_OK;
		}
	}//if end
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		printDebug(1, "[ err ]: GetFingerImage err!");
		return FAILURE;
	}

	//Operation is successful!
	printDebug(1, "[ ok ]: [03]GetFingerImage.....OK!");
	if(DATABUFF != NULL)
	{
		free(DATABUFF);
		DATABUFF = NULL;
		//memory free OK!
	}
	return SUCCESS;
}


INT IdentifyByImage(INT imageSize, INT *userID, INT *index, U8 * data)
{
	INT ret = -1;
	INT Param = imageSize;
	INT Flag = 0;
	INT len = 0;


	//1. send command
	ret = Cmd.SendCmd(MD_IDENTIFY_IMAGE_X, Param, Flag, NULL, 0);
	if(ret < 0)
	{
		return FAILURE;
	}
	ret = Cmd.RecvCmd(MD_IDENTIFY_IMAGE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. send image
	if(Flag == FLAG_OK)
	{
		SendExtDataPro(MD_IDENTIFY_IMAGE_X, &Param, &Flag, TIMEOUT, data, imageSize);
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	//return info
	*userID = (Param & 0xffff);
	*index = (Param >> 16) & 0x0f;

	//Operation is successful!
	printDebug(1, "[ ok ]: [04]IdentifyByImage.....OK!");
	return SUCCESS;
}

INT EnrollTemplateByImage (INT userID, U8 *data, INT dataSize)
{
	INT ret = -1;
	INT Param = userID;
	INT Flag = 0;
	INT len = 4;

	INT size = dataSize;
	//1. send command

	ret = Cmd.SendCmd(MD_ENROLL_IMAGE_X, Param, Flag, (U8 *)&size, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	ret = Cmd.RecvCmd(MD_ENROLL_IMAGE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. send data
	if(Flag == FLAG_OK)
	{
		SendExtDataPro(MD_ENROLL_IMAGE_X, &Param, &Flag, TIMEOUT, data, dataSize);
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [02]EnrollTemplateByImage.....OK!");
	return SUCCESS;
}

INT GetUser(INT userID,  CHAR *name,  CHAR *password, U16 *secLevel,
		U32 *PIN2, U8 *privilege, U8 *fingerprintNum, U8 *Card)
{
	INT ret = -1;
	INT Param = userID;
	INT Flag = 0;
	INT len = sizeof(tUser);

	pUser data = NULL;
	SHORT PacketNum = 0;
	SHORT Index = 0;

	//send data
	ret = Cmd.SendCmd(MD_READ_USER, Param, Flag, NULL, 0);
	if(ret < 0)
	{
		return FAILURE;
	}
	if(data != NULL)
	{
		free(data);
		data = NULL;
	}
	data = (pUser)malloc(sizeof(tUser));
	if(NULL == data)
	{
		//malloc Err
		return FAILURE;
	}

	//recv data
	ret = Cmd.extRecvCmd(MD_READ_USER, &Flag, &PacketNum, &Index, (U8 *)data, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	memcpy(name, data->Name, sizeof(data->Name));
	memcpy(password, data->Password, sizeof(data->Password));
	memcpy(Card, data->Card, sizeof(data->Card));
	*secLevel = data->SecLevel;
	*PIN2 = data->PIN2;
	*privilege = data->Privilege;
	*fingerprintNum = data->Fpnum;

	//Operation is successful!
	printDebug(1, "[ ok ]: [18]GetUser.....OK!");
	if(data != NULL)
	{
		free(data);
		data = NULL;
	}
	return SUCCESS;
}

INT SetTemplates(INT userID, INT flag, U8 *data, INT dataSize)
{
	INT ret = -1;
	INT Param = userID;
	INT Flag = 0;
	INT len = dataSize;


	switch(flag)
	{
		case 0:
			Flag = 0;
			break;
		case 1:
			Flag = 0x84;
			break;
		default:
			//Parameter err...
			return FAILURE;
	}
	if(NULL == data)
	{
		//Data is null...
		return FAILURE;
	}

	//send data
	ret = Cmd.SendCmd(MD_ENROLL_TMP, Param, Flag, data, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//recv data
	ret = Cmd.RecvCmd(MD_ENROLL_TMP, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	return SUCCESS;
}

INT GetTime (INT *Year, INT *Month, INT *Day,  INT *Hour, INT *Minute, INT *Second)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT time = 0;


	//date init
	*Year = *Month = *Day = *Hour = *Minute = *Second = 0;
	//1. send data
	ret = Cmd.SendCmd(MD_GET_TIME, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//2. recv data
	ret = Cmd.RecvCmd(MD_GET_TIME, &Flag, &Param, (U8*)&time, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//3. Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}

	//4. return time
	*Year = (Param & 0xff);
	Param >>= 8;
	*Month = (Param & 0xff);
	Param >>= 8;
	*Day = (Param & 0xff);

	*Hour = (time & 0xff);
	time >>= 8;
	*Minute = (time & 0xff);
	time >>= 8;
	*Second = (time & 0xff);

	//Operation is successful!
	printDebug(1, "[ ok ]: [10]GetTime.....OK!");
	return SUCCESS;
}

INT SetTime(INT Year, INT Month, INT Day,  INT Hour, INT Minute, INT Second)
{
	INT ret = -1;
	INT time = 0;
	INT Param = 0;
	INT Flag = 0;

	INT len = 4;

	//1. time data
	if(timeCheck(Year, Month, Day, Hour, Minute, Second) < 0)
	{
		printDebug(1, "[ err ]: Time Parameter err!");
		return FAILURE;
	}

	/*DD<<16 | MM<<8 | YY    ss<<16 | mm<<8 | hh*/
	Year -= 2000;
	Param |= (Day & 0xff);
	Param <<= 8;
	Param |= (Month & 0xff);
	Param <<= 8;
	Param |= (Year & 0xff);

	time |= (Second & 0xff);
	time <<= 8;
	time |= (Minute & 0xff);
	time <<= 8;
	time |= (Hour & 0xff);

	SetTime_loop = 1;//1:output setTime datapack  [on]
	//2. send data
	ret = Cmd.SendCmd(MD_SET_TIME, Param, Flag, (U8*)&time, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	SetTime_loop = 0;//1:output setTime datapack [off]
	//3. recv data
	ret = Cmd.RecvCmd(MD_SET_TIME, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//4. Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [09]SetTime.....OK!");
	return SUCCESS;
}

INT DeleteTemplates(INT userID, INT index, INT flag)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;

	switch(flag)
	{
		case 0:
			Flag = 0;
			break;
		case 1:
			Flag = 0x77;
			break;
		default:
			printDebug(1, "[ err ]: flag Parameter err!");
			return FAILURE;
	}
	if((index < 0) || (index > 9))
	{
		printDebug(1, "[ err ]: Index Parameter err!");
		return FAILURE;
	}
	else
	{
		Param = (userID & 0xFFFF) | ((index & 0x0F) << 16);
	}

	//send data
	ret = Cmd.SendCmd(MD_DEL_TMP, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//recv data
	ret = Cmd.RecvCmd(MD_DEL_TMP, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [21]DeleteTemplates.....OK!");
	return SUCCESS;
}

INT SetParameter (INT value, INT flag)
{
	INT ret = -1;
	INT Param = value;
	INT Flag = 0;
	INT len = 0;

	if(!ParameterSelect(flag))
	{
		return FAILURE;
	}
	Flag = flag;
	//send data
	ret = Cmd.SendCmd(MD_SYS_WP, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_SYS_WP, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [13]SetParameter.....OK!");
	return SUCCESS;
}

INT GetParameter(INT flag, INT *value)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;

	if(!ParameterSelect(flag))
	{
		return FAILURE;
	}
	Flag = flag;

	//send data
	ret = Cmd.SendCmd(MD_SYS_RP, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_SYS_RP, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	*value = Param;
	//Operation is successful!
	printDebug(1, "[ ok ]: [12]GetParameter.....OK!");
	return SUCCESS;
}

INT DeleteUser(INT userID)
{
	INT ret = -1;
	INT Param = userID;
	INT Flag = 0;

	INT len = 0;

	//send data
	ret = Cmd.SendCmd(MD_DELETE_USER, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DELETE_USER, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [16]DeleteUser.....OK!");
	return SUCCESS;
}

INT DeleteAllUsers(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;

	//send data
	ret = Cmd.SendCmd(MD_DEL_ALL_USER, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DEL_ALL_USER, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [15]DeleteAllUsers.....OK!");
	return SUCCESS;
}

INT SaveParameter(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;

	//send data
	ret = Cmd.SendCmd(MD_SYS_SP, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_SYS_SP, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [14]SaveParameter.....OK!");
	return SUCCESS;
}

INT ClearDB (VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	
	//send data
	ret = Cmd.SendCmd(MD_DEL_DB, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DEL_DB, &Flag, &Param, NULL, &len, TIMEOUT*3);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		//Operation failure...
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [7]ClearDB.....OK!");
	return SUCCESS;
}

INT DeleteAllTemplates(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;

	//send data
	ret = Cmd.SendCmd(MD_DEL_ALL_TMP, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DEL_ALL_TMP, &Flag, &Param, NULL, &len, TIMEOUT *2);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [24]DeleteAllTemplates.....OK!");
	return SUCCESS;
}

INT Verify (INT userID)
{
	INT ret = -1;
	INT Param = userID;
	INT Flag = FLAG_OK;

	INT len = 0;
	
	//send data
	ret = Cmd.SendCmd(MD_VERIFY_SCAN, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//recv data
	ret = Cmd.RecvCmd(MD_VERIFY_SCAN, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [05]Verify.....OK!");
	return SUCCESS;
}


INT DeleteAllLogs (VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;
	
	//send data
	ret = Cmd.SendCmd(MD_DEL_ALOG, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DEL_ALOG, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [06]DeleteAllLogs.....OK!");
	return SUCCESS;
}

VOID TimeAnalyse(U32 date, U32 time, INT *Year, INT *Month, INT *Day,  INT *Hour, INT *Minute, INT *Second)
{
	*Year = (date & 0xff);
	date >>= 8;
	*Month = (date & 0xff);
	date >>= 8;
	*Day = (date & 0xff);

	*Hour = (time & 0xff);
	time >>= 8;
	*Minute = (time & 0xff);
	time >>= 8;
	*Second = (time & 0xff);
}

INT ScanTemplate(U8 *data)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;

	U8 *buffer = NULL;

	buffer = malloc(TEMPLATES_SIZE);
	if(buffer == NULL)
	{
		printDebug(1, "[ err ]: Failed to malloc!");
		return FAILURE;
	}
	memset(buffer, 0, TEMPLATES_SIZE);
	//1. send command
	ret = Cmd.SendCmd(MD_SCAN_TEMPLATE, Param, Flag, NULL, len);
	if (ret < 0) {
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_SCAN_TEMPLATE, &Flag, &Param, buffer, &len, TIMEOUT);
	if (ret < 0) {
		return FAILURE;
	}

	//Operation OK?
	if (Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//return data
	memcpy(data, buffer, len);

	//Operation is successful!
	//output data
	printDebug(1, "[ ok ]: [25]ScanTemplate.....OK!");
	if(buffer != NULL)
	{
		free(buffer);
		buffer = NULL;
	}
	return SUCCESS;
}

INT Disconnect(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;


	//send data
	ret = Cmd.SendCmd(MD_DISCONNECT, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	//recv data
	ret = Cmd.RecvCmd(MD_DISCONNECT, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	printDebug(1, "[ ok ]: [66]Disconnect.....OK!");
	sleep(1);

	//free cache and close interface
	if(Final())
	{
		//"Final OK!"
	}
	return SUCCESS;
}

INT Reset(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;


	//send data
	ret = Cmd.SendCmd(MD_RESET, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_RESET, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	printDebug(1, "[ ok ]: [28]Reset.....OK!");
	return SUCCESS;
}

INT DisableDevice(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;


	//send data
	ret = Cmd.SendCmd(MD_DISABLEDEVICE, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_DISABLEDEVICE, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	printDebug(1, "[ ok ]: [27]Disable_Device.....OK!");
	return SUCCESS;
}

INT EnableDevice(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;

	INT len = 0;


	//send data
	ret = Cmd.SendCmd(MD_ENABLEDEVICE, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv data
	ret = Cmd.RecvCmd(MD_ENABLEDEVICE, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	printDebug(1, "[ ok ]: [26]Enable_Device.....OK!");
	return SUCCESS;
}

INT Upgrade (U8 *fw, INT size)
{
	INT ret = -1;
	INT Param = size;
	INT Flag = 0;
	INT len = 0;

	//1. send command
	ret = Cmd.SendCmd(MD_UPDATE_FW, Param, Flag, NULL, 0);
	if(ret < 0)
	{
		return FAILURE;
	}

	ret = Cmd.RecvCmd(MD_UPDATE_FW, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. send image
	if(Flag == FLAG_OK)
	{
		SendExtDataPro(MD_UPDATE_FW, &Param, &Flag, TIMEOUT, fw, size);
	}
	if(Dev_num == DEV_232)
	{
		pd_msleep(10);
	}
	ret = Cmd.RecvCmd(MD_UPDATE_FW, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [29]Upgrade.....OK!");
	return SUCCESS;
}

INT UploadTemplateFile (U8 *fw, INT size)
{
	INT ret = -1;
	INT Param = size;
	INT Flag = 0;
	INT len = 0;

	//1. send command
	ret = Cmd.SendCmd(MD_WT_FILE_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}

	ret = Cmd.RecvCmd(MD_WT_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT*2);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. send image
	if(Flag == FLAG_OK)
	{
		SendExtDataPro(MD_WT_FILE_X, &Param, &Flag, TIMEOUT, fw, size);
	}
	if(Dev_num == DEV_232)
	{
		pd_msleep(10);
	}
	ret = Cmd.RecvCmd(MD_WT_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [30]WriteAllTemplateFile.....OK!");
	return SUCCESS;
}

INT UploadUserFile(U8 *fw, INT size)
{
	INT ret = -1;
	INT Param = size;
	INT Flag = 0;
	INT len = 0;

	//1. send command
	ret = Cmd.SendCmd(MD_WU_FILE_X, Param, Flag, NULL, 0);
	if(ret < 0)
	{
		return FAILURE;
	}

	ret = Cmd.RecvCmd(MD_WU_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT*3);
	if(ret < 0)
	{
		return FAILURE;
	}
	//2. send image
	if(Flag == FLAG_OK)
	{
		SendExtDataPro(MD_WU_FILE_X, &Param, &Flag, TIMEOUT, fw, size);
	}
	if(Dev_num == DEV_232)
	{
		pd_msleep(100);
	}
	ret = Cmd.RecvCmd(MD_WU_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	//Operation is successful!
	printDebug(1, "[ ok ]: [31]WriteAllUserFile.....OK!");
	return SUCCESS;
}

INT DownloadUserFile(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;
	FILE *fp = NULL;
	U8 *data = NULL;

	if(NULL == (fp = fopen("UserFile", "wb")))
	{
		printDebug(1, "[ err ]: fopen AllUserDataFile Fail!");
		return FAILURE;
	}
	//1. send command
	ret = Cmd.SendCmd(MD_RU_FILE_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_RU_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	size = Param;

	if(Flag == FLAG_OK)
	{
		//2. recv extData
		data = malloc(size);
		if(data == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(data, 0, size);
		SetExtDataBuffer(size, data);
		RecvExtDataPro(MD_RU_FILE_X, TIMEOUT);	//RECV OK!
	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	// 3.write file
	fwrite(data, size, 1, fp);
	printDebug(1, "[ ok ]: [32]ReadAllUserFile.....OK!");
	// 4. free
	if(data != NULL)
	{
		free(data);
		data = NULL;
	}
	fclose(fp);
	return SUCCESS;
}

INT DownloadTemplatesFile(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;
	INT size = 0;
	FILE *fp = NULL;
	U8 *data = NULL;

	if(NULL == (fp = fopen("TemplatesFile", "wb")))
	{
		printDebug(1, "[ err ]: fopen AllUserDataFile Fail!");
		return FAILURE;
	}
	//1. send command
	ret = Cmd.SendCmd(MD_RT_FILE_X, Param, Flag, NULL, len);
	if(ret < 0)
	{
		return FAILURE;
	}
	//recv command
	ret = Cmd.RecvCmd(MD_RT_FILE_X, &Flag, &Param, NULL, &len, TIMEOUT);
	if(ret < 0)
	{
		return FAILURE;
	}
	size = Param;

	if(Flag == FLAG_OK)
	{
		//2. recv extData
		data = malloc(size);
		if(data == NULL)
		{
			printDebug(1, "[ err ]: Failed to malloc!");
			return FAILURE;
		}
		memset(data, 0, size);
		SetExtDataBuffer(size, data);
		RecvExtDataPro(MD_RT_FILE_X, TIMEOUT);	//RECV OK!
	}//if end

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		Err_flag = Flag;
		return FAILURE;
	}
	// 3.write file
	fwrite(data, size, 1, fp);
	printDebug(1, "[ ok ]: [33]ReadAllTemplatesFile.....OK!");
	// 4. free
	if(data != NULL)
	{
		free(data);
		data = NULL;
	}
	fclose(fp);
	return SUCCESS;
}

INT FreeScan(VOID)
{
	INT ret = -1;
	INT Param = 0;
	INT Flag = 0;
	INT len = 0;

	ret = Cmd.RecvCmd(MD_IDENTIFY_FREE, &Flag, &Param, NULL, &len, 150);
	if(ret < 0)
	{
		return FAILURE;
	}

	//Operation OK?
	if(Flag != FLAG_OK)
	{
		//Operation failure...
		Err_flag = Flag;
		return -1;
	}

	//return info

	//Operation is successful!
	printDebug(1, "[ ok ]: FreeScan.....OK!");
	return Param;
}

int GetLastErr(void)
{
	return Err_flag;
}

int Clear(void)
{
	if(Dev_num == DEV_USB) {
		Recv_data();
	}
	return SUCCESS;
}

char *GetSDKVersion(void)
{
	return SDKVERSION;
}

unsigned long GetTicks(void)
{
	return pd_get_ticks();
}
